@@ -123,8 +123,6 @@ EXCLUDE = (
   "cmd/tools/unflatten.c",
   "contrib/diffimg/diffimg.c",
   "contrib/pangotest/pangotest.c",
-  "contrib/prune/generic_list.c",
-  "contrib/prune/generic_list.h",
   "contrib/prune/prune.c",
   "doc/libgraph/agmemread.c",
   "doc/libgraph/sccmap.c",
@@ -577,8 +575,6 @@ EXCLUDE = (
   "lib/sparse/BinaryHeap.h",
   "lib/sparse/DotIO.c",
   "lib/sparse/DotIO.h",
-  "lib/sparse/IntStack.c",
-  "lib/sparse/IntStack.h",
   "lib/sparse/LinkedList.c",
   "lib/sparse/LinkedList.h",
   "lib/sparse/QuadTree.c",
