@@ -10,13 +10,14 @@
 
 
 #include "config.h"
-
+#include <cgraph/alloc.h>
 /* TODO:
  * If cut point is in exactly 2 blocks, expand block circles to overlap
  * especially in the case where one block is the sole child of the other.
  */
 
 #include	<circogen/blockpath.h>
+#include	<math.h>
 
 /* getRotation:
  * The function determines how much the block should be rotated
@@ -43,12 +44,9 @@
  * Thus, 
  *    alpha = theta + M_PI/2 - phi - arcsin((l/R)*(cos phi))
  */
-static double
-getRotation(block_t * sn, Agraph_t * g, double x, double y, double theta)
-{
+static double getRotation(block_t *sn, double x, double y, double theta) {
     double mindist2;
     Agraph_t *subg;
-    /* Agedge_t* e; */
     Agnode_t *n, *closest_node, *neighbor;
     nodelist_t *list;
     double len2, newX, newY;
@@ -92,7 +90,6 @@ getRotation(block_t * sn, Agraph_t * g, double x, double y, double theta)
 	}
     }
 
-    /* if((neighbor != closest_node) && !ISPARENT(neighbor)) { */
     if (neighbor != closest_node) {
 	double rho = sn->rad0;
 	double r = sn->radius - rho;
@@ -225,19 +222,16 @@ setInfo (posinfo_t* p0, posinfo_t* p1, double delta)
 
     t /= 2*delta*p0->minRadius*p1->minRadius;
 
-    if (t < 1)
-	t = 1;
+    t = fmax(t, 1);
 
-    if (t > p0->scale)
-	p0->scale = t;
-    if (t > p1->scale)
-	p1->scale = t;
+    p0->scale = fmax(p0->scale, t);
+    p1->scale = fmax(p1->scale, t);
 }
 
 /* positionChildren:
  */
 static void
-positionChildren (Agraph_t* g, posinfo_t* pi, posstate * stp, int length, double min_dist)
+positionChildren(posinfo_t *pi, posstate *stp, int length, double min_dist)
 {
     block_t *child;
     double childAngle, childRadius, incidentAngle;
@@ -301,7 +295,7 @@ positionChildren (Agraph_t* g, posinfo_t* pi, posstate * stp, int length, double
 	 * should return the theta value if there was a rotation else zero
 	 */
 
-	rotateAngle = getRotation(child, g, deltaX, deltaY, childAngle);
+	rotateAngle = getRotation(child, deltaX, deltaY, childAngle);
 	applyDelta(child, deltaX, deltaY, rotateAngle);
 
 	if (length == 1) {
@@ -332,8 +326,7 @@ positionChildren (Agraph_t* g, posinfo_t* pi, posstate * stp, int length, double
  * Finally, positionChildren is called to do the actual positioning.
  * If length is 1, keeps track of minimum and maximum child angle.
  */
-static double
-position(Agraph_t * g, int childCount, int length, nodelist_t * path,
+static double position(int childCount, int length, nodelist_t *path,
 	 block_t * sn, double min_dist)
 {
     nodelistitem_t *item;
@@ -343,7 +336,7 @@ position(Agraph_t * g, int childCount, int length, nodelist_t * path,
     double maxRadius = 0.0;
     double angle;
     double theta = 0.0;
-    posinfo_t* parents = N_NEW(childCount, posinfo_t);
+    posinfo_t* parents = gv_calloc(childCount, sizeof(posinfo_t));
     int num_parents = 0;
     posinfo_t* next;
     posinfo_t* curr;
@@ -398,7 +391,7 @@ position(Agraph_t * g, int childCount, int length, nodelist_t * path,
     }
 
     for (i = 0; i < num_parents; i++) {
-	positionChildren (g, parents + i, &state, length, min_dist);
+	positionChildren(parents + i, &state, length, min_dist);
     }
 
     free (parents);
@@ -444,8 +437,7 @@ static void doBlock(Agraph_t * g, block_t * sn, double min_dist)
 
     /* attach children */
     if (childCount > 0)
-	centerAngle =
-	    position(g, childCount, length, longest_path, sn, min_dist);
+	centerAngle = position(childCount, length, longest_path, sn, min_dist);
 
     if ((length == 1) && (BLK_PARENT(sn))) {
 	sn->parent_pos = centerAngle;
