@@ -8,17 +8,27 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-
+#include <assert.h>
 #include <common/render.h>
 #include <common/htmltable.h>
 #include "htmlparse.h"
 #include <common/htmllex.h>
 #include <cdt/cdt.h>
 #include <ctype.h>
+#include <cgraph/alloc.h>
 #include <cgraph/strcasecmp.h>
+#include <cgraph/strview.h>
+#include <cgraph/tokenize.h>
+#include <cgraph/unused.h>
 #include <limits.h>
+#include <stddef.h>
 
 #ifdef HAVE_EXPAT
+#ifdef _WIN32
+// ensure that the expat functions get the correct storage class
+// declarations also on MinGW
+#define XML_USE_MSC_EXTENSIONS 1
+#endif
 #include <expat.h>
 #endif
 
@@ -30,18 +40,18 @@ typedef struct {
 #ifdef HAVE_EXPAT
     XML_Parser parser;
 #endif
-    char* ptr;			/* input source */
-    int tok;			/* token type   */
-    agxbuf* xb;			/* buffer to gather T_string data */
-    agxbuf  lb;			/* buffer for translating lexical data */
-    char warn;			/* set if warning given */
-    char error;			/* set if error given */
-    char inCell;		/* set if in TD to allow T_string */
-    char mode;			/* for handling artificial <HTML>..</HTML> */
-    char *currtok;		/* for error reporting */
-    char *prevtok;		/* for error reporting */
-    int currtoklen;
-    int prevtoklen;
+    char* ptr;         // input source
+    int tok;           // token type
+    agxbuf* xb;        // buffer to gather T_string data
+    agxbuf lb;         // buffer for translating lexical data
+    int warn;          // set if warning given
+    int error;         // set if error given
+    char inCell;       // set if in TD to allow T_string
+    char mode;         // for handling artificial <HTML>..</HTML>
+    char *currtok;     // for error reporting
+    char *prevtok;     // for error reporting
+    size_t currtoklen;
+    size_t prevtoklen;
 } lexstate_t;
 static lexstate_t state;
 
@@ -164,21 +174,21 @@ static int portfn(htmldata_t * p, char *v)
 static int stylefn(htmldata_t * p, char *v)
 {
     int rv = 0;
-    char* tk;
-    char* buf = strdup (v);
-    for (tk = strtok (buf, DELIM); tk; tk = strtok (NULL, DELIM)) {
-	if (!strcasecmp(tk, "ROUNDED")) p->style |= ROUNDED;
-	else if (!strcasecmp(tk, "RADIAL")) p->style |= RADIAL;
-	else if(!strcasecmp(tk,"SOLID")) p->style &= ~(DOTTED|DASHED);
-	else if(!strcasecmp(tk,"INVISIBLE") || !strcasecmp(tk,"INVIS")) p->style |= INVISIBLE;
-	else if(!strcasecmp(tk,"DOTTED")) p->style |= DOTTED;
-	else if(!strcasecmp(tk,"DASHED")) p->style |= DASHED;
+    for (tok_t t = tok(v, DELIM); !tok_end(&t); tok_next(&t)) {
+	strview_t tk = tok_get(&t);
+	if (strview_case_str_eq(tk, "ROUNDED")) p->style |= ROUNDED;
+	else if (strview_case_str_eq(tk, "RADIAL")) p->style |= RADIAL;
+	else if (strview_case_str_eq(tk,"SOLID")) p->style &= (unsigned short)~(DOTTED|DASHED);
+	else if (strview_case_str_eq(tk,"INVISIBLE") ||
+	         strview_case_str_eq(tk,"INVIS")) p->style |= INVISIBLE;
+	else if (strview_case_str_eq(tk,"DOTTED")) p->style |= DOTTED;
+	else if (strview_case_str_eq(tk,"DASHED")) p->style |= DASHED;
 	else {
-	    agerr(AGWARN, "Illegal value %s for STYLE - ignored\n", tk);
+	    agerr(AGWARN, "Illegal value %.*s for STYLE - ignored\n", (int)tk.size,
+	          tk.data);
 	    rv = 1;
 	}
     }
-    free (buf);
     return rv;
 }
 
@@ -270,9 +280,9 @@ static int cellborderfn(htmltbl_t * p, char *v)
 {
     long u;
 
-    if (doInt(v, "CELLSBORDER", 0, SCHAR_MAX, &u))
+    if (doInt(v, "CELLBORDER", 0, SCHAR_MAX, &u))
 	return 1;
-    p->cb = (unsigned char) u;
+    p->cb = (signed char)u;
     return 0;
 }
 
@@ -549,7 +559,7 @@ doAttrs(void *tp, attr_item * items, int nel, char **atts, char *s)
     while ((name = *atts++) != NULL) {
 	val = *atts++;
 	key.name = name;
-	ip = (attr_item *) bsearch(&key, items, nel, ISIZE, (bcmpfn) icmp);
+	ip = bsearch(&key, items, nel, ISIZE, (bcmpfn) icmp);
 	if (ip)
 	    state.warn |= ip->action(tp, val);
 	else {
@@ -575,12 +585,13 @@ static htmlimg_t *mkImg(char **atts)
     return img;
 }
 
-static textfont_t *mkFont(GVC_t *gvc, char **atts, int flags, int ul)
-{
+static textfont_t *mkFont(GVC_t *gvc, char **atts, unsigned char flags) {
     textfont_t tf = {NULL,NULL,NULL,0.0,0,0};
 
     tf.size = -1.0;		/* unassigned */
-    tf.flags = flags;
+    enum { FLAGS_MAX = (1 << GV_TEXTFONT_FLAGS_WIDTH) - 1  };
+    assert(flags <= FLAGS_MAX);
+    tf.flags = (unsigned char)(flags & FLAGS_MAX);
     if (atts)
 	doAttrs(&tf, font_items, sizeof(font_items) / ISIZE, atts, "<FONT>");
 
@@ -625,28 +636,28 @@ static void startElement(void *user, const char *name, char **atts)
 	htmllval.cell = mkCell(atts);
 	state.tok = T_cell;
     } else if (strcasecmp(name, "FONT") == 0) {
-	htmllval.font = mkFont(gvc, atts, 0, 0);
+	htmllval.font = mkFont(gvc, atts, 0);
 	state.tok = T_font;
     } else if (strcasecmp(name, "B") == 0) {
-	htmllval.font = mkFont(gvc, 0, HTML_BF, 0);
+	htmllval.font = mkFont(gvc, 0, HTML_BF);
 	state.tok = T_bold;
     } else if (strcasecmp(name, "S") == 0) {
-	htmllval.font = mkFont(gvc, 0, HTML_S, 0);
+	htmllval.font = mkFont(gvc, 0, HTML_S);
 	state.tok = T_s;
     } else if (strcasecmp(name, "U") == 0) {
-	htmllval.font = mkFont(gvc, 0, HTML_UL, 1);
+	htmllval.font = mkFont(gvc, 0, HTML_UL);
 	state.tok = T_underline;
     } else if (strcasecmp(name, "O") == 0) {
-	htmllval.font = mkFont(gvc, 0, HTML_OL, 1);
+	htmllval.font = mkFont(gvc, 0, HTML_OL);
 	state.tok = T_overline;
     } else if (strcasecmp(name, "I") == 0) {
-	htmllval.font = mkFont(gvc, 0, HTML_IF, 0);
+	htmllval.font = mkFont(gvc, 0, HTML_IF);
 	state.tok = T_italic;
     } else if (strcasecmp(name, "SUP") == 0) {
-	htmllval.font = mkFont(gvc, 0, HTML_SUP, 0);
+	htmllval.font = mkFont(gvc, 0, HTML_SUP);
 	state.tok = T_sup;
     } else if (strcasecmp(name, "SUB") == 0) {
-	htmllval.font = mkFont(gvc, 0, HTML_SUB, 0);
+	htmllval.font = mkFont(gvc, 0, HTML_SUB);
 	state.tok = T_sub;
     } else if (strcasecmp(name, "BR") == 0) {
 	mkBR(atts);
@@ -667,6 +678,8 @@ static void startElement(void *user, const char *name, char **atts)
 
 static void endElement(void *user, const char *name)
 {
+    (void)user;
+
     if (strcasecmp(name, "TABLE") == 0) {
 	state.tok = T_end_table;
 	state.inCell = 1;
@@ -727,6 +740,8 @@ static void endElement(void *user, const char *name)
  */
 static void characterData(void *user, const char *s, int length)
 {
+    (void)user;
+
     int i, cnt = 0;
     unsigned char c;
 
@@ -747,7 +762,7 @@ int initHTMLlexer(char *src, agxbuf * xb, htmlenv_t *env)
 {
 #ifdef HAVE_EXPAT
     state.xb = xb;
-    agxbinit (&state.lb, SMALLBUF, NULL);
+    state.lb = (agxbuf){0};
     state.ptr = src;
     state.mode = 0;
     state.warn = 0;
@@ -785,6 +800,15 @@ int clearHTMLlexer()
 #endif
 }
 
+/// \p agxbput, but assume that source and destination may overlap
+static UNUSED void agxbput_move(agxbuf *dst, const char *src) {
+  // we cannot call `agxbput` itself because it calls `memcpy`, thereby
+  // implicitly assuming that source and destination do not overlap
+  char *src_copy = gv_strdup(src);
+  agxbput(dst, src_copy);
+  free(src_copy);
+}
+
 #ifdef HAVE_EXPAT
 /* eatComment:
  * Given first character after open comment, eat characters
@@ -869,19 +893,24 @@ static char *findNext(char *s, agxbuf* xb)
 static void protect_rsqb(agxbuf *xb) {
 
   // if the buffer is empty, we have nothing to do
-  if (xb->buf == xb->ptr) {
+  if (agxblen(xb) == 0) {
     return;
   }
 
-  // if the buffer does not end in a ], we have nothing to do
-  if (xb->ptr[-1] != ']') {
+  // check the last character and if it is not ], we have nothing to do
+  char *data = agxbuse(xb);
+  size_t size = strlen(data);
+  assert(size > 0);
+  if (data[size - 1] != ']') {
+    agxbput_move(xb, data);
     return;
   }
 
-  // rewind over the ]
-  --xb->ptr;
+  // truncate ] and write back the remaining prefix
+  data[size - 1] = '\0';
+  agxbput_move(xb, data);
 
-  // write an XML-escaped version of ]
+  // write an XML-escaped version of ] as a replacement
   agxbput(xb, "&#93;");
 }
 #endif
@@ -1019,9 +1048,9 @@ static void printTok(int tok)
 	s = "<unknown>";
     }
     if (tok == T_string) {
-	fprintf(stderr, "%s \"", s);
-	fwrite(agxbstart(state.xb), 1, agxblen(state.xb), stderr);
-	fprintf(stderr, "\"\n");
+	const char *token_text = agxbuse(state.xb);
+	fprintf(stderr, "%s \"%s\"\n", s, token_text);
+	agxbput_move(state.xb, token_text);
     } else
 	fprintf(stderr, "%s\n", s);
 }
@@ -1036,7 +1065,7 @@ int htmllex()
 
     char *s;
     char *endp = 0;
-    int len, llen;
+    size_t len, llen;
     int rv;
 
     state.tok = 0;
@@ -1056,7 +1085,7 @@ int htmllex()
 		len = strlen(s);
 	    } else {
 		endp = findNext(s,&state.lb);
-		len = endp - s;
+		len = (size_t)(endp - s);
 	    }
 	}
 
@@ -1066,10 +1095,13 @@ int htmllex()
 	state.prevtoklen = state.currtoklen;
 	state.currtok = s;
 	state.currtoklen = len;
-	if ((llen = agxblen(&state.lb)))
-	    rv = XML_Parse(state.parser, agxbuse(&state.lb),llen, 0);
-	else
-	    rv = XML_Parse(state.parser, s, len, len ? 0 : 1);
+	if ((llen = (size_t)agxblen(&state.lb))) {
+	    assert(llen <= (size_t)INT_MAX && "XML token too long for expat API");
+	    rv = XML_Parse(state.parser, agxbuse(&state.lb), (int)llen, 0);
+	} else {
+	    assert(len <= (size_t)INT_MAX && "XML token too long for expat API");
+	    rv = XML_Parse(state.parser, s, (int)len, len ? 0 : 1);
+	}
 	if (rv == XML_STATUS_ERROR) {
 	    if (!state.error) {
 		agerr(AGERR, "%s in line %d \n",
