@@ -15,6 +15,7 @@
 
 #include <assert.h>
 #include <cgraph/alloc.h>
+#include <common/boxes.h>
 #include <dotgen/dot.h>
 #include <limits.h>
 #include <math.h>
@@ -55,58 +56,6 @@
 	ED_to_orig(newp) = old; \
 }
 
-// a dynamically expanding array of boxes
-typedef struct {
-  boxf *data;
-  size_t size;
-  size_t capacity;
-} boxes_t;
-
-/** Add an entry to the end of a boxes array.
- *
- * This may expand the array if it is not already large enough to contain the
- * new element.
- *
- * \param boxes Array to append to.
- * \param item Element to append.
- */
-static void boxes_append(boxes_t *boxes, boxf item) {
-
-  assert(boxes != NULL);
-
-  // do we need to expand the array?
-  if (boxes->size == boxes->capacity) {
-    size_t c = boxes->capacity == 0 ? 128 : (boxes->capacity * 2);
-    boxes->data = grealloc(boxes->data, c * sizeof(boxes->data[0]));
-    boxes->capacity = c;
-  }
-
-  boxes->data[boxes->size] = item;
-  ++boxes->size;
-}
-
-/** Remove all entries from a boxes array.
- *
- * \param boxes Array to clear.
- */
-static void boxes_clear(boxes_t *boxes) {
-  assert(boxes != NULL);
-  boxes->size = 0;
-}
-
-/** Deallocate memory associated with a boxes array.
- *
- * Following a call to this function, the array is reusable as if it had just
- * been initialized.
- *
- * \param boxes Array to deallocate.
- */
-static void boxes_free(boxes_t *boxes) {
-  assert(boxes != NULL);
-  free(boxes->data);
-  memset(boxes, 0, sizeof(*boxes));
-}
-
 typedef struct {
     int LeftBound, RightBound, Splinesep, Multisep;
     boxf* Rank_box;
@@ -2175,16 +2124,11 @@ completeregularpath(path * P, edge_t * first, edge_t * last,
  */
 static boxf makeregularend(boxf b, int side, double y)
 {
-    boxf newb;
-    switch (side) {
-    case BOTTOM:
-	newb = boxfof(b.LL.x, y, b.UR.x, b.LL.y);
-	break;
-    case TOP:
-	newb = boxfof(b.LL.x, b.UR.y, b.UR.x, y);
-	break;
-    }
-    return newb;
+  assert(side == BOTTOM || side == TOP);
+  if (side == BOTTOM) {
+    return boxfof(b.LL.x, y, b.UR.x, b.LL.y);
+  }
+  return boxfof(b.LL.x, b.UR.y, b.UR.x, y);
 }
 
 #ifndef DONT_WANT_ANY_ENDPOINT_PATH_REFINEMENT
