@@ -29,9 +29,12 @@
 #define FDP_PRIVATE 1
 
 #include "config.h"
+#include <assert.h>
 #include <limits.h>
 #include <inttypes.h>
 #include <assert.h>
+#include <cgraph/alloc.h>
+#include <cgraph/list.h>
 #include <fdpgen/tlayout.h>
 #include <math.h>
 #include <neatogen/neatoprocs.h>
@@ -41,6 +44,7 @@
 #include <fdpgen/clusteredges.h>
 #include <fdpgen/dbg.h>
 #include <stddef.h>
+#include <stdbool.h>
 
 typedef struct {
     graph_t*  rootg;  /* logical root; graph passed in to fdp_layout */
@@ -182,9 +186,9 @@ static node_t *mkDeriveNode(graph_t * dg, char *name)
     node_t *dn;
 
     dn = agnode(dg, name,1);
-    agbindrec(dn, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);	//node custom data
-    ND_alg(dn) = NEW(dndata);	/* free in freeDeriveNode */
-    ND_pos(dn) = N_GNEW(GD_ndim(dg), double);
+    agbindrec(dn, "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);	//node custom data
+    ND_alg(dn) = gv_alloc(sizeof(dndata)); // free in freeDeriveNode
+    ND_pos(dn) = gv_calloc(GD_ndim(dg), sizeof(double));
     /* fprintf (stderr, "Creating %s\n", dn->name); */
     return dn;
 }
@@ -271,36 +275,7 @@ static void evalPositions(graph_t * g, graph_t* rootg)
     }
 }
 
-#define CL_CHUNK 10
-
-typedef struct {
-    graph_t **cl;
-    int sz;
-    int cnt;
-} clist_t;
-
-static void initCList(clist_t * clist)
-{
-    clist->cl = 0;
-    clist->sz = 0;
-    clist->cnt = 0;
-}
-
-/* addCluster:
- * Append a new cluster to the list.
- * NOTE: cl[0] is empty. The clusters are in cl[1..cnt].
- * Normally, we increase the array when cnt == sz.
- * The test for cnt > sz is necessary for the first time.
- */
-static void addCluster(clist_t * clist, graph_t * subg)
-{
-    clist->cnt++;
-    if (clist->cnt >= clist->sz) {
-	clist->sz += CL_CHUNK;
-	clist->cl = RALLOC(clist->sz, clist->cl, graph_t *);
-    }
-    clist->cl[clist->cnt] = subg;
-}
+DEFINE_LIST(clist, graph_t*)
 
 #define BSZ 1000
 
@@ -380,7 +355,7 @@ static void addEdge(edge_t * de, edge_t * e)
     edge_t **el;
 
     el = (edge_t **) (ED_to_virt(de));
-    el = ALLOC(cnt + 1, el, edge_t *);
+    el = gv_recalloc(el, cnt, cnt + 1, sizeof(edge_t*));
     el[cnt] = e;
     ED_to_virt(de) = (edge_t *) el;
     ED_count(de)++;
@@ -427,12 +402,12 @@ static graph_t *deriveGraph(graph_t * g, layout_info * infop)
     infop->gid++;
 
     dg = agopen("derived", Agstrictdirected,NULL);
-    agbindrec(dg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);
+    agbindrec(dg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
     GD_alg(dg) = NEW(gdata);	/* freed in freeDeriveGraph */
 #ifdef DEBUG
     GORIG(dg) = g;
 #endif
-    GD_ndim(dg) = GD_ndim(g);
+    GD_ndim(dg) = GD_ndim(agroot(g));
 
     /* Copy attributes from g.
      */
@@ -502,7 +477,7 @@ static graph_t *deriveGraph(graph_t * g, layout_info * infop)
 		de = agedge(dg, tl, hd, NULL,1);
 	    else
 		de = agedge(dg, hd, tl, NULL,1);
-	    agbindrec(de, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);
+	    agbindrec(de, "Agedgeinfo_t", sizeof(Agedgeinfo_t), true);
 	    ED_dist(de) = ED_dist(e);
 	    ED_factor(de) = ED_factor(e);
 	    /* fprintf (stderr, "edge %s -- %s\n", tl->name, hd->name); */
@@ -523,7 +498,7 @@ static graph_t *deriveGraph(graph_t * g, layout_info * infop)
 	int sz = NPORTS(g);
 
 	/* freed in freeDeriveGraph */
-	PORTS(dg) = pq = N_NEW(sz + 1, bport_t);
+	PORTS(dg) = pq = gv_calloc(sz + 1, sizeof(bport_t));
 	sz = 0;
 	while (pp->e) {
 	    m = DNODE(pp->n);
@@ -536,7 +511,7 @@ static graph_t *deriveGraph(graph_t * g, layout_info * infop)
 		    de = agedge(dg, m, dn, NULL,1);
 		else
 		    de = agedge(dg, dn, m, NULL,1);
-		agbindrec(de, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);
+		agbindrec(de, "Agedgeinfo_t", sizeof(Agedgeinfo_t), true);
 		ED_dist(de) = ED_dist(pp->e);
 		ED_factor(de) = ED_factor(pp->e);
 		addEdge(de, pp->e);
@@ -585,7 +560,6 @@ static int ecmp(const void *v1, const void *v2)
  */
 static erec *getEdgeList(node_t * n, graph_t * g)
 {
-    erec *erecs;
     int deg = DEG(n);
     int i;
     double dx, dy;
@@ -593,7 +567,7 @@ static erec *getEdgeList(node_t * n, graph_t * g)
     node_t *m;
 
     /* freed in expandCluster */
-    erecs = N_NEW(deg + 1, erec);
+    erec *erecs = gv_calloc(deg + 1, sizeof(erec));
     i = 0;
     for (e = agfstedge(g, n); e; e = agnxtedge(g, e, n)) {
 	if (aghead(e) == n)
@@ -705,14 +679,13 @@ static graph_t *expandCluster(node_t * n, graph_t * cg)
     erec *ep;
     erec *next;
     graph_t *sg = ND_clust(n);
-    bport_t *pp;
     int sz = WDEG(n);
     int idx = 0;
     double bnd;
 
     if (sz != 0) {
 	/* freed in cleanup_subgs */
-	pp = N_NEW(sz + 1, bport_t);
+	bport_t *pp = gv_calloc(sz + 1, sizeof(bport_t));
 
 	/* create sorted list of edges of n */
 	es = ep = getEdgeList(n, cg);
@@ -898,12 +871,12 @@ static int layout(graph_t * g, layout_info * infop)
      * How to combine parts, especially with disparate components?
      */
     if (c_cnt > 1) {
-	boolean *bp;
+	bool *bp;
 	if (pinned) {
-	    bp = N_NEW(c_cnt, boolean);
-	    bp[0] = TRUE;
+	    bp = gv_calloc(c_cnt, sizeof(bool));
+	    bp[0] = true;
 	} else
-	    bp = 0;
+	    bp = NULL;
 	infop->pack.fixed = bp;
 	pts = putGraphs(c_cnt, cc, NULL, &infop->pack);
 	free(bp);
@@ -996,12 +969,13 @@ static void
 mkClusters (graph_t * g, clist_t* pclist, graph_t* parent)
 {
     graph_t* subg;
-    clist_t  list;
+    clist_t  list = {0};
     clist_t* clist;
 
     if (pclist == NULL) {
+	// [0] is empty. The clusters are in [1..cnt].
+	clist_append(&list, NULL);
 	clist = &list;
-	initCList(clist);
     }
     else
 	clist = pclist;
@@ -1009,12 +983,12 @@ mkClusters (graph_t * g, clist_t* pclist, graph_t* parent)
     for (subg = agfstsubg(g); subg; subg = agnxtsubg(subg))
 	{
 	if (!strncmp(agnameof(subg), "cluster", 7)) {
-	    agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);
+	    agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
 	    GD_alg(subg) = NEW(gdata);	/* freed in cleanup_subgs */
-	    GD_ndim(subg) = GD_ndim(parent);
+	    GD_ndim(subg) = GD_ndim(agroot(parent));
 	    LEVEL(subg) = LEVEL(parent) + 1;
 	    GPARENT(subg) = parent;
-	    addCluster(clist, subg);
+	    clist_append(clist, subg);
 	    mkClusters(subg, NULL, subg);
 	}
 	else {
@@ -1022,18 +996,23 @@ mkClusters (graph_t * g, clist_t* pclist, graph_t* parent)
 	}
     }
     if (pclist == NULL) {
-	GD_n_cluster(g) = list.cnt;
-	if (list.cnt)
-	    GD_clust(g) = RALLOC(list.cnt + 1, list.cl, graph_t*);
+	assert(clist_size(&list) - 1 <= INT_MAX);
+	GD_n_cluster(g) = (int)(clist_size(&list) - 1);
+	if (clist_size(&list) > 1) {
+	    clist_shrink_to_fit(&list);
+	    GD_clust(g) = clist_detach(&list);
+	} else {
+	    clist_free(&list);
+	}
     }
 }
 
 static void fdp_init_graph(Agraph_t * g)
 {
-    setEdgeType (g, ET_LINE);
+    setEdgeType (g, EDGETYPE_LINE);
     GD_alg(g) = NEW(gdata);	/* freed in cleanup_graph */
-    GD_ndim(g) = late_int(g, agattr(g,AGRAPH, "dim", NULL), 2, 2);
-    Ndim = GD_ndim(g) = MIN(GD_ndim(g), MAXDIM);
+    GD_ndim(agroot(g)) = late_int(g, agattr(g,AGRAPH, "dim", NULL), 2, 2);
+    Ndim = GD_ndim(agroot(g)) = MIN(GD_ndim(agroot(g)), MAXDIM);
 
     mkClusters (g, NULL, g);
     fdp_initParams(g);
@@ -1067,18 +1046,18 @@ fdpSplines (graph_t * g)
     int trySplines = 0;
     int et = EDGE_TYPE(g);
 
-    if (et > ET_ORTHO) {
-	if (et == ET_COMPOUND) {
-	    trySplines = splineEdges(g, compoundEdges, ET_SPLINE);
+    if (et > EDGETYPE_ORTHO) {
+	if (et == EDGETYPE_COMPOUND) {
+	    trySplines = splineEdges(g, compoundEdges, EDGETYPE_SPLINE);
 	    /* When doing the edges again, accept edges done by compoundEdges */
 	    if (trySplines)
 		Nop = 2;
 	}
-	if (trySplines || et != ET_COMPOUND) {
+	if (trySplines || et != EDGETYPE_COMPOUND) {
 	    if (HAS_CLUST_EDGE(g)) {
 		agerr(AGWARN,
 		      "splines and cluster edges not supported - using line segments\n");
-		et = ET_LINE;
+		et = EDGETYPE_LINE;
 	    } else {
 		spline_edges1(g, et);
 	    }
@@ -1100,7 +1079,7 @@ void fdp_layout(graph_t * g)
     }
     neato_set_aspect(g);
 
-    if (EDGE_TYPE(g) != ET_NONE) fdpSplines (g); 
+    if (EDGE_TYPE(g) != EDGETYPE_NONE) fdpSplines (g);
 
     gv_postprocess(g, 0);
     PSinputscale = save_scale;
